package com.ldodds.twinkle.ui.components.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JMenuItem;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;

import com.ldodds.twinkle.ui.components.ComponentRegistry;

/**
 * Twinkle Edit Menu
 * @author Leigh Dodds
 *
 */
public class EditMenu extends TwinkleMenu
{

	private final class RedoAction implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
		    try
		    {
		    	ComponentRegistry.registry.getCurrentFrame().getUndoManager().redo();
		    } catch (CannotRedoException ex)
		    {
		    	ComponentRegistry.registry.getMainWindow().getToolkit().beep();
		    }
		}
	}

	private static final class UndoAction implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
		    try
		    {
		    	ComponentRegistry.registry.getCurrentFrame().getUndoManager().undo();
		    } catch (CannotUndoException ex)
		    {
		    	ComponentRegistry.registry.getMainWindow().getToolkit().beep();
		    }
		}
	}

	private static Pattern commentStart = Pattern.compile("# ?");
	
	private static final class ToggleCommentAction implements ActionListener {
		public void actionPerformed(ActionEvent e){
			JTextArea ta = ComponentRegistry.registry.getCurrentFrame().getQueryField();
			Document doc = ta.getDocument();
			Element root = doc.getDefaultRootElement();
			int startSel = ta.getSelectionStart();
			int endSel=ta.getSelectionEnd();
//			System.err.println("ToggleCommentAction pos:"+startSel+":"+endSel);
			try {
				int startLine = ta.getLineOfOffset(startSel);
				int endLine = ta.getLineOfOffset(endSel);
				if(startSel!=endSel && root.getElement(endLine).getStartOffset()==endSel)endLine--;
//				System.err.println("ToggleCommentAction lines:"+startLine+":"+endLine);
				for(int l=endLine;l>=startLine;l--){
					int offset=root.getElement(l).getStartOffset();
//					System.err.println("ToggleCommentAction offset:"+offset);
					String c=ta.getText(offset,root.getElement(l).getEndOffset()-offset);
					Matcher m = commentStart.matcher(c);
					if(m.lookingAt())
						doc.remove(offset,m.end());
					else 
						doc.insertString(offset,"# ",null);
				}
			} catch (BadLocationException ex){
				System.err.println("bad location:"+startSel+":"+endSel);
			}
		}
	}

	private static final class IncFontSizeAction implements ActionListener {
		public void actionPerformed(ActionEvent e){
			ComponentRegistry.registry.getCurrentFrame().incrementFontSize(2);
		}
	}
	private static final class DecFontSizeAction implements ActionListener {
		public void actionPerformed(ActionEvent e){
			ComponentRegistry.registry.getCurrentFrame().incrementFontSize(-2);
		}
	}

	public EditMenu() {
		super("Edit", KeyEvent.VK_E);
		
		JMenuItem undoMenuItem = 
			createMenuItem("Undo", KeyEvent.VK_U, 
					"/actions/edit-undo.png", 
					KeyStroke.getKeyStroke(KeyEvent.VK_Z, META_KEY), 
					new UndoAction());
		this.add(undoMenuItem);

		JMenuItem redoMenuItem = 
			createMenuItem("Redo", KeyEvent.VK_R, 
					"/actions/edit-redo.png", 
					KeyStroke.getKeyStroke(KeyEvent.VK_Y, META_KEY), 
					new RedoAction());
		this.add(redoMenuItem);		

		this.addSeparator();
		
		JMenuItem cutMenuItem = 
			createMenuItem("Cut", KeyEvent.VK_X, 
					"/actions/edit-cut.png", 
					KeyStroke.getKeyStroke(KeyEvent.VK_X, META_KEY), 
					ComponentRegistry.registry.getCurrentFrame().getQueryField().getActionMap().get(DefaultEditorKit.cutAction) );
		this.add(cutMenuItem);		

		JMenuItem copyMenuItem = 
			createMenuItem("Copy", KeyEvent.VK_C, 
					"/actions/edit-copy.png", 
					KeyStroke.getKeyStroke(KeyEvent.VK_C, META_KEY), 
					ComponentRegistry.registry.getCurrentFrame().getQueryField().getActionMap().get(DefaultEditorKit.copyAction) );
		this.add(copyMenuItem);		

		JMenuItem pasteMenuItem = 
			createMenuItem("Paste", KeyEvent.VK_V, 
					"/actions/edit-paste.png", 
					KeyStroke.getKeyStroke(KeyEvent.VK_V, META_KEY), 
					ComponentRegistry.registry.getCurrentFrame().getQueryField().getActionMap().get(DefaultEditorKit.pasteAction) );
		this.add(pasteMenuItem);		

		this.addSeparator();

		JMenuItem toggleCommentMenuItem = 
			createMenuItem("Toggle comment", KeyEvent.VK_T,
					null, 
					KeyStroke.getKeyStroke(KeyEvent.VK_T,META_KEY), 
					new ToggleCommentAction() );
		this.add(toggleCommentMenuItem);		
				
		JMenuItem incFontSizeMenuItem = 
			createMenuItem("Larger font", KeyEvent.VK_PLUS,
					null, 
					KeyStroke.getKeyStroke(KeyEvent.VK_PLUS,META_KEY), 
					new IncFontSizeAction() );
		this.add(incFontSizeMenuItem);		

		JMenuItem decFontSizeMenuItem = 
			createMenuItem("Smaller font", KeyEvent.VK_MINUS,
					null, 
					KeyStroke.getKeyStroke(KeyEvent.VK_MINUS,META_KEY), 
					new DecFontSizeAction() );
		this.add(decFontSizeMenuItem);		
	}

}
