package com.ldodds.twinkle.ui.components.menu;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.swing.AbstractAction;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import com.hp.hpl.jena.assembler.JA;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.vocabulary.RDF;
import com.hp.hpl.jena.vocabulary.RDFS;
import com.ldodds.twinkle.ui.components.ComponentRegistry;
import com.ldodds.twinkle.ui.components.QueryInterface;
import com.ldodds.util.Icons;

public class TwinkleMenu extends JMenu
{
    protected static int META_KEY = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

	public TwinkleMenu(String title, int mnemonic) {
		super(title);
		this.setMnemonic(mnemonic);
	}

	protected JMenuItem createMenuItem(String label, 
			int mnemonic, String icon, KeyStroke accelerator, ActionListener listener) {
        JMenuItem menuItem = new JMenuItem(label);
        menuItem.setMnemonic(mnemonic);
        if (icon != null) {
            menuItem.setIcon( 
            		Icons.createImageIcon(icon, label) );                	
        }
        if (accelerator != null) {
        	menuItem.setAccelerator(accelerator);	
        }
        if (listener != null) {
        	menuItem.addActionListener(listener);	
        }
        
		return menuItem;
	}
	
	private final class InsertInQueryAction extends AbstractAction
	{
		private String content;
		
		public InsertInQueryAction(String content) {
			this.content=content;
		}

		public void actionPerformed(ActionEvent e) {
			QueryInterface queryInterface = ComponentRegistry.registry.getCurrentFrame();
			queryInterface.insertInQueryString(content);
		}
	}

	// add Menu element and sort them in order of label using a SortedMap
	void addMenuElements(JMenuItem prefixMenu, RDFNode contentNode){
		Model config = ComponentRegistry.registry.getConfig();
		ResIterator mappings = config.listSubjectsWithProperty(RDF.type, contentNode);
		SortedMap<String,String> map = new TreeMap<String,String>();
		while ( mappings.hasNext() ) {
			Resource r = mappings.nextResource();
			if (contentNode==JA.ContentItem){
				if (r.hasProperty(RDFS.label) &&  r.hasProperty(JA.content)) {
					map.put(r.getProperty(RDFS.label).getObject().toString(), 
							r.getProperty(JA.content).getObject().toString()+"\n");
				}
			} else if (contentNode==JA.PrefixMapping){
				if (r.hasProperty(JA.prefix) && r.hasProperty(JA.namespace) && r.hasProperty(RDFS.label) ) {
					map.put(r.getProperty(RDFS.label).getObject().toString(), 
							"PREFIX " + r.getProperty(JA.prefix).getObject() + 
							": <" + r.getProperty(JA.namespace).getObject() + ">\n");
					}
			}
		}
		for(Map.Entry<String, String> e: map.entrySet()){
			JMenuItem item = new JMenuItem(e.getKey());
			item.addActionListener( new InsertInQueryAction(e.getValue()) );
			prefixMenu.add( item );			
		}
	}
}
